% Comparing the convergence of LMS, NLMS, and RLS
% MTMS, august 2011
% Equalization of a QPSK signal.

clear
close all

% number of iterations
Nit=5000;

% Channel 
h=[-0.2+0.3i; -0.5+0.4i; 0.7-0.6i; 0.4+0.3i; 0.2+0.1i]; 
% See [Chen, 2003]

% Signal-to-noise ratio in dB
SNR=30;

%Equalizer length
M=15;

% delay
delay=9;

%adaptation parameters 
mu=1e-3;
mut=0.08;
lb=0.997;
lbi=1/lb;

% Initialization
winit=zeros(1,M);

% Number of independent runs
Navg=1000;

e2lms=zeros(1,Nit);
e2rls=zeros(1,Nit);
e2nlms=zeros(1,Nit);

for nn=1:Navg
nn
% transmitted signal
aR=2*round(rand(1,Nit))-1;
aI=2*round(rand(1,Nit))-1;
a=aR+j*aI;

% Signal at the receiver
x=filter(h,1,a);
Px=sum(real(x).^2)/Nit;
Pn=Px*10^(-SNR/10);
noise=sqrt(Pn)*(randn(1,Nit)+j*randn(1,Nit));
x=x+noise;

% desired signal, 
d=[zeros(1,delay) a(1:end-delay)];


xM=zeros(M,1);
wlms=zeros(Nit+1,M);
wnlms=zeros(Nit+1,M);
wrls=zeros(Nit+1,M);
wlms(1,:)=winit;
wnlms(1,:)=winit;
wrls(1,:)=winit;
I=eye(M);
P=I;
elms=zeros(1,Nit);
erls=zeros(1,Nit);
enlms=zeros(1,Nit);

ylms=zeros(1,Nit);
yrls=zeros(1,Nit);
ynlms=zeros(1,Nit);
Nx=0;

for n=1:Nit
       Nx=Nx+abs(x(n))^2-abs(xM(M))^2;
       xM=[x(n);xM(1:M-1)];
       
       ylms(n)=conj(wlms(n,:))*xM;
       elms(n)=d(n)-ylms(n);
        
       ynlms(n)=conj(wnlms(n,:))*xM;
       enlms(n)=d(n)-ynlms(n);
       
       yrls(n)=conj(wrls(n,:))*xM;
       erls(n)=d(n)-yrls(n);
       
       wlms(n+1,:)=wlms(n,:)+mu*conj(elms(n))*xM.';
                   
       wnlms(n+1,:)=wnlms(n,:)+mut*conj(enlms(n))*xM.'/(1e-5+Nx);
       
       aux=P*xM;
       k=aux/(lb+xM'*aux);
       wrls(n+1,:)=wrls(n,:)+k.'*conj(erls(n));
       P=lbi*(I-k*xM')*P;
       
end;

e2lms=e2lms+abs(elms).^2;
e2rls=e2rls+abs(erls).^2;
e2nlms=e2nlms+abs(enlms).^2;
end


figure(1)
subplot(221)
plot(ylms(0.9*Nit:Nit),'.r')
title('a) LMS')
xlabel('Re(y)')
ylabel('Im(y)')
grid
subplot(222)
plot(ynlms(0.9*Nit:Nit),'.g')
title('b) NLMS')
xlabel('Re(y)')
ylabel('Im(y)')
grid
subplot(223)
plot(yrls(0.9*Nit:Nit),'.b')
grid
title('c) RLS')
xlabel('Re(y)')
ylabel('Im(y)')

figure(2)
subplot(221)
plot(10*log10(e2lms/Navg),'r')
hold on
plot(10*log10(e2nlms/Navg),'g')
plot(10*log10(e2rls/Navg),'b')
grid
xlabel('iterations')
ylabel('MSE (dB)')
axis([0 Nit -20 5])

